# Copyright 2012 Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
import unittest

from service_registry.client import Client, HeartBeater

TOKENS = ['6bc8d050-f86a-11e1-a89e-ca2ffe480b20']

EXPECTED_METADATA = \
    {'region': 'dfw',
     'port': '3306',
     'ip': '127.0.0.1',
     'version': '5.5.24-0ubuntu0.12.04.1 (Ubuntu)'}


class FarscapeClientTests(unittest.TestCase):
    def setUp(self):
        self.client = Client('user',
                             'api_key',
                             'http://127.0.0.1:8881/')

    def authenticate(fn):
        def wrapped(*args, **kwargs):
            name = 'service_registry.client.BaseClient._authenticate'
            with mock.patch(name) as _authenticate:
                _authenticate.return_value = {'X-Auth-Token': 'auth_token',
                                              'X-Tenant-Id': 'tenant_id'}
                fn(*args, **kwargs)
        return wrapped

    @authenticate
    def test_get_limits(self):
        expected = \
            {'rate':
                {'/.*': {'window': '24.0 hours', 'used': 0, 'limit': 500000}},
             'resource': {}}

        result = self.client.account.get_limits()
        self.assertEqual(result, expected)

    @authenticate
    def test_create_session(self):
        response_body = {'token': TOKENS[0]}
        result = self.client.sessions.create(15)

        self.assertEqual(result[0], response_body)
        self.assertEqual(result[1], 'sessionId')
        self.assertTrue(isinstance(result[2], HeartBeater))
        self.assertEqual(result[2].heartbeat_interval, 12.0)
        self.assertEqual(result[2].heartbeat_timeout, 15)
        self.assertEqual(result[2].next_token, TOKENS[0])

    @authenticate
    def test_heartbeat_session(self):
        result = self.client.sessions.heartbeat('sessionId', 'someToken')

        self.assertEqual(result, {'token': TOKENS[0]})

    @authenticate
    def test_create_service(self):
        result = self.client.services.create('sessionId', 'dfw1-db1')

        self.assertEqual(result, 'dfw1-db1')

    @authenticate
    def test_get_service(self):
        result = self.client.services.get('dfw1-db1')
        self.assertEqual(result['id'], 'dfw1-db1')
        self.assertEqual(result['session_id'], 'sessionId')
        self.assertEqual(result['tags'], ['db', 'mysql'])
        self.assertEqual(result['metadata'], EXPECTED_METADATA)

    @authenticate
    def test_list_services(self):
        result = self.client.services.list()

        self.assertEqual(result['values'][0]['id'], 'dfw1-api')
        self.assertEqual(result['values'][0]['session_id'], 'sessionId')
        self.assertTrue('tags' in result['values'][0])
        self.assertTrue('metadata' in result['values'][0])
        self.assertEqual(result['values'][1]['id'], 'dfw1-db1')
        self.assertEqual(result['values'][1]['session_id'], 'sessionId')
        self.assertEqual(result['values'][1]['tags'],
                         ['db', 'mysql'])
        self.assertEqual(result['values'][1]['metadata'],
                         EXPECTED_METADATA)
        self.assertTrue('metadata' in result)

    @authenticate
    def test_list_for_tag(self):
        result = self.client.services.list_for_tag('db')

        self.assertEqual(result['values'][0]['id'], 'dfw1-db1')
        self.assertEqual(result['values'][0]['session_id'], 'sessionId')
        self.assertEqual(result['values'][0]['tags'],
                         ['db', 'mysql'])
        self.assertEqual(result['values'][0]['metadata'],
                         EXPECTED_METADATA)
        self.assertTrue('metadata' in result)

    @authenticate
    def test_get_sessions(self):
        result = self.client.sessions.list()

        self.assertEqual(result['values'][0]['id'], 'sessionId')
        self.assertEqual(result['values'][0]['heartbeat_timeout'], 30)
        self.assertTrue('metadata' in result['values'][0])
        self.assertTrue('last_seen' in result['values'][0])
        self.assertTrue('metadata' in result)

    @authenticate
    def test_get_session(self):
        result = self.client.sessions.get('sessionId')

        self.assertEqual(result['id'], 'sessionId')
        self.assertEqual(result['heartbeat_timeout'], 30)
        self.assertTrue('metadata' in result)
        self.assertTrue('last_seen' in result)

    @authenticate
    def test_list_configuration(self):
        result = self.client.configuration.list()

        self.assertEqual(result['values'][0]['id'], 'configId')
        self.assertEqual(result['values'][0]['value'], 'test value 123456')
        self.assertTrue('metadata' in result)

    @authenticate
    def test_get_configuration(self):
        result = self.client.configuration.get('configId')

        self.assertEqual(result['id'], 'configId')
        self.assertEqual(result['value'], 'test value 123456')

if __name__ == '__main__':
    unittest.main()



# Copyright 2012 Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import subprocess
import signal
import time
import socket
import errno
import atexit
from os.path import join as pjoin


# From https://github.com/Kami/python-yubico-client/blob/master/tests/utils.py

def waitForStartUp(process, pid, address, timeout=10):
    # connect to it, with a timeout in case something went wrong
    start = time.time()
    while time.time() < start + timeout:
        try:
            s = socket.create_connection(address)
            s.close()
            break
        except:
            time.sleep(0.1)
    else:
        # see if process is still alive
        process.poll()

        if pid and process.returncode is None:
            os.kill(pid, signal.SIGKILL)
        raise RuntimeError("Couldn't connect to server; aborting test")


class ProcessRunner(object):
    def setUp(self, *args, **kwargs):
        # clean up old.
        p = self.getPid()
        if p is not None:
            try:
                # remember, process may already be dead.
                os.kill(p, 9)
                time.sleep(0.01)
            except:
                pass

    def tearDown(self, *args, **kwargs):
        spid = self.getPid()
        if spid:
            max_wait = 1
            os.kill(spid, signal.SIGTERM)
            slept = 0
            while (slept < max_wait):
                time.sleep(0.5)
                if not self.isAlive(spid):
                    if os.path.exists(self.pid_fname):
                        os.unlink(self.pid_fname)
                    break
                slept += 0.5
            if (slept > max_wait and self.isAlive(spid)):
                os.kill(spid, signal.SIGKILL)
                if os.path.exists(self.pid_fname):
                    os.unlink(self.pid_fname)
                raise Exception('Server did not shut down correctly')
        else:
            print 'Unable to locate pid file (%s)!' % self.pid_fname

    def isAlive(self, pid):
        try:
            os.kill(pid, 0)
            return 1
        except OSError, err:
            return err.errno == errno.EPERM

    def getPid(self):
        if self.process:
            return self.process.pid
        elif os.path.exists(self.pid_fname):
            return int(open(self.pid_fname, 'r').read())
        return None


class MockAPIServerRunner(ProcessRunner):
    def __init__(self, port=8881):
        self.port = port

    def setUp(self, *args, **kwargs):
        self.cwd = os.getcwd()
        self.process = None
        self.base_dir = pjoin(self.cwd)
        self.pid_fname = pjoin(self.cwd, 'mock_api_server.pid')
        self.log_path = pjoin(self.cwd, 'mock_api_server.log')

        super(MockAPIServerRunner, self).setUp(*args, **kwargs)
        script = pjoin(os.path.dirname(__file__), 'mock_http_server.py')

        with open(self.log_path, 'a+') as log_fp:
            port_arg = '%s --port=%s' % (script, self.port)
            fixtures_dir_arg = \
                '--fixtures-dir=service_registry/test/fixtures/response/'
            args = '%s %s' % (port_arg, fixtures_dir_arg)
            self.process = subprocess.Popen(args,
                                            shell=True,
                                            cwd=self.base_dir,
                                            stdout=log_fp,
                                            stderr=log_fp)
            waitForStartUp(self.process,
                           self.getPid(),
                           ('127.0.0.1', self.port), 10)
        atexit.register(self.tearDown)



# Copyright 2012 Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from copy import deepcopy
try:
        import simplejson as json
except:
        import json

from dateutil import parser
import httplib
from libcloud.common.types import InvalidCredsError, MalformedResponseError
from libcloud.compute.drivers.rackspace import RackspaceNodeDriver
import random
import requests
from time import mktime, sleep, time

US_AUTH_URL = 'https://identity.api.rackspacecloud.com/v2.0/tokens'
UK_AUTH_URL = 'https://lon.identity.api.rackspacecloud.com/v2.0/tokens'
DEFAULT_AUTH_URLS = {'us': US_AUTH_URL,
                     'uk': UK_AUTH_URL}
DEFAULT_API_URL = 'https://dfw.registry.api.rackspacecloud.com/v1.0/'
MAX_HEARTBEAT_TIMEOUT = 30
MAX_401_RETRIES = 1


ACCEPTABLE_STATUS_CODES = {'GET': (httplib.OK,),
                           'POST': (httplib.OK, httplib.CREATED),
                           'PUT': (httplib.NO_CONTENT,),
                           'DELETE': (httplib.NO_CONTENT,)}


class BaseClient(object):
    def __init__(self, base_url, username, api_key, region):
        self.base_url = base_url
        self.username = username
        self.api_key = api_key
        self.auth_headers = None
        self.auth_token_expires = None
        self.region = region

        valid_regions = DEFAULT_AUTH_URLS.keys()
        if region not in valid_regions:
            raise ValueError('Invalid region %s. Valid regions are: %s' % (
                             region, ', '.join(valid_regions)))

        auth_url = DEFAULT_AUTH_URLS[region]

        if not auth_url.endswith('/'):
            auth_url += '/'

        self.auth_url = auth_url

    def get_id_from_url(self, url):
        return url.split('/')[-1]

    def request(self, method, path, options=None, payload=None,
                heartbeater=None, re_authenticate=False, retry_count=0):
        self.auth_headers = self._authenticate(force=re_authenticate)
        tenant_id = self.auth_headers['X-Tenant-Id']
        request_url = self.base_url + tenant_id + path

        if method not in ['GET', 'POST', 'PUT', 'DELETE']:
            raise ValueError('Invalid method: %s' % (method))

        data = json.dumps(payload) if payload else None
        request_kwargs = {'method': method.lower(), 'url': request_url,
                          'headers': self.auth_headers, 'params': options,
                          'data': data}

        if retry_count < MAX_401_RETRIES:
            retry_count += 1
            r = requests.request(**request_kwargs)

            if r.status_code == httplib.UNAUTHORIZED:
                return self.request(method=method, path=path, options=options,
                                    payload=payload, heartbeater=heartbeater,
                                    re_authenticate=True,
                                    retry_count=retry_count)
        else:
            raise APIError('API returned 401')

        def _check_status_code(status_code, method):
            if status_code not in ACCEPTABLE_STATUS_CODES[method]:
                data = r.json
                raise ValidationError(type=data['type'], code=data['code'],
                                      message=data['message'],
                                      txnId=data['txnId'],
                                      details=data['details'])

        if method == 'GET':
            _check_status_code(r.status_code, 'GET')

            return r.json
        elif method == 'POST':
            _check_status_code(r.status_code, 'POST')

            if 'heartbeat' in path:
                return r.json

            id_from_url = self.get_id_from_url(r.headers['location'])

            if 'services' in path:
                return id_from_url

            heartbeater.session_id = id_from_url
            heartbeater.next_token = r.json['token']

            return r.json, id_from_url, heartbeater
        elif method == 'PUT':
            _check_status_code(r.status_code, 'PUT')

            return True
        elif method == 'DELETE':
            _check_status_code(r.status_code, 'DELETE')

            return True

    def _authenticate(self, force=False):
        if self.auth_headers:
            current_time = time()
            if not force and (self.auth_token_expires and
                             (self.auth_token_expires < current_time)):
                return self.auth_headers
        try:
            driver = RackspaceNodeDriver(self.username, self.api_key,
                                         ex_force_auth_url=self.auth_url,
                                         ex_force_auth_version='2.0',
                                         ex_force_service_region=self.region)
            driver.connection._populate_hosts_and_request_paths()
            auth_token = driver.connection.auth_token
            tenant_id = driver.connection.request_path.split('/')[-1]
            expires = driver.connection.auth_token_expires
            expires_datetime = parser.parse(expires)
            self.auth_token_expires = \
                mktime(expires_datetime.timetuple())
            return {'X-Auth-Token': auth_token,
                    'X-Tenant-Id': tenant_id}
        except (InvalidCredsError, MalformedResponseError):
            raise InvalidCredentialsError('The username or password you'
                                          ' entered is incorrect. Please'
                                          ' try again.')


class SessionsClient(BaseClient):
    def __init__(self, base_url, username, api_key, region):
        super(SessionsClient, self).__init__(base_url, username,
                                             api_key, region)
        self.sessions_path = '/sessions'
        self.base_url = base_url
        self.username = username
        self.api_key = api_key
        self.region = region

    def create(self, heartbeat_timeout, payload=None):
        path = self.sessions_path
        payload = deepcopy(payload) if payload else {}
        payload['heartbeat_timeout'] = heartbeat_timeout

        heartbeater = HeartBeater(self.base_url,
                                  self.username,
                                  self.api_key,
                                  self.region,
                                  None,
                                  heartbeat_timeout)

        return self.request('POST',
                            path,
                            payload=payload,
                            heartbeater=heartbeater)

    def list(self):
        path = self.sessions_path

        return self.request('GET', path)

    def get(self, session_id):
        path = '%s/%s' % (self.sessions_path, session_id)

        return self.request('GET', path)

    def heartbeat(self, session_id, token):
        path = '%s/%s/heartbeat' % (self.sessions_path, session_id)
        payload = {'token': token}

        return self.request('POST', path, payload=payload)

    def update(self, session_id, payload):
        path = '%s/%s' % (self.sessions_path, session_id)

        return self.request('PUT', path, payload=payload)


class EventsClient(BaseClient):
    def __init__(self, base_url, username, api_key, region):
        super(EventsClient, self).__init__(base_url, username,
                                           api_key, region)
        self.events_path = '/events'

    def list(self, marker=None):
        options = None
        if marker:
            options = {'marker': marker}

        return self.request('GET', self.events_path, options=options)


class ServicesClient(BaseClient):
    def __init__(self, base_url, username, api_key, region):
        super(ServicesClient, self).__init__(base_url, username,
                                             api_key, region)
        self.services_path = '/services'

    def list(self):
        return self.request('GET', self.services_path)

    def list_for_tag(self, tag):
        options = {'tag': tag}

        return self.request('GET', self.services_path, options=options)

    def get(self, service_id):
        path = '%s/%s' % (self.services_path, service_id)

        return self.request('GET', path)

    def create(self, session_id, service_id, payload=None):
        payload = deepcopy(payload) if payload else {}
        payload['session_id'] = session_id
        payload['id'] = service_id

        return self.request('POST', self.services_path, payload=payload)

    def update(self, service_id, payload):
        path = '%s/%s' % (self.services_path, service_id)

        return self.request('PUT', path, payload=payload)

    def remove(self, service_id):
        path = '%s/%s' % (self.services_path, service_id)

        return self.request('DELETE', path)

    def register(self, session_id, service_id, payload=None, retry_delay=2):
        retry_count = MAX_HEARTBEAT_TIMEOUT / retry_delay
        success = False
        retry_counter = 0
        last_err = None
        result = None

        def do_register(success, result, retry_counter, last_err):
            if success and (retry_counter < retry_count):
                return result

            elif (not success) and (retry_counter == retry_count):
                return last_err
            try:
                result = self.create(session_id, service_id, payload)
                success = True

                return do_register(success, result, retry_counter, last_err)
            except ValidationError as e:
                last_err = e.args[0]
                if last_err.type == 'serviceWithThisIdExists':
                    retry_counter += 1
                    sleep(retry_delay)

                    return do_register(success, result, retry_counter,
                                       last_err)
                else:
                    return last_err

        return do_register(success, result, retry_counter, last_err)


class ConfigurationClient(BaseClient):
    def __init__(self, base_url, username, api_key, region):
        super(ConfigurationClient, self).__init__(base_url, username,
                                                  api_key, region)
        self.configuration_path = '/configuration'

    def list(self):
        return self.request('GET', self.configuration_path)

    def get(self, configuration_id):
        path = '%s/%s' % (self.configuration_path, configuration_id)

        return self.request('GET', path)

    def set(self, configuration_id, value):
        path = '%s/%s' % (self.configuration_path, configuration_id)
        payload = {'value': value}

        return self.request('PUT', path, payload=payload)

    def remove(self, configuration_id):
        path = '%s/%s' % (self.configuration_path, configuration_id)

        return self.request('DELETE', path)


class AccountClient(BaseClient):
    def __init__(self, base_url, username, api_key, region):
        super(AccountClient, self).__init__(base_url, username,
                                            api_key, region)
        self.limits_path = '/limits'

    def get_limits(self):
        return self.request('GET', self.limits_path)


class HeartBeater(BaseClient):
    def __init__(self, base_url, username, api_key, region,
                 session_id, heartbeat_timeout):
        """
        HeartBeater will start heartbeating a session once start() is called,
        and stop heartbeating the session when stop() is called.

        @param base_url:  The base Cloud Registry URL.
        @type base_url: C{str}
        @param username: Rackspace username.
        @type username: C{str}
        @param api_key: Rackspace API key.
        @type api_key: C{str}
        @param session_id: The ID of the session to heartbeat.
        @type session_id: C{str}
        @param heartbeat_timeout: The amount of time after which a session will
        time out if a heartbeat is not received.
        @type heartbeat_timeout: C{int}
        """
        super(HeartBeater, self).__init__(base_url, username, api_key, region)
        self.session_id = session_id
        self.heartbeat_timeout = heartbeat_timeout
        self.heartbeat_interval = self._calculate_interval(heartbeat_timeout)
        self.next_token = None
        self._stopped = False

    def _calculate_interval(self, heartbeat_timeout):
        if heartbeat_timeout < 15:
            return heartbeat_timeout * 0.6
        else:
            return heartbeat_timeout * 0.8

    def _start_heartbeating(self):
        path = '/sessions/%s/heartbeat' % self.session_id
        payload = {'token': self.next_token}

        if self._stopped:
            return

        interval = self.heartbeat_interval

        if interval > 5:
            interval = interval + random.randrange(-3, 1)

        sleep(interval)

        result = self.request('POST', path, payload=payload)

        self.next_token = result['token']

        self._start_heartbeating()

    def start(self):
        """
        Start heartbeating the session. Will continue to heartbeat
        until stop() is called.
        """
        return self._start_heartbeating()

    def stop(self):
        """
        Stop heartbeating the session.
        """
        self._stopped = True


class ValidationError(Exception):
    def __init__(self, type, code, message, txnId, details):
        self.type = type
        self.code = code
        self.message = message
        self.txnId = txnId
        self.details = details

    def __str__(self):
        return ('<ValidationError type=%s, code=%s, txnId=%s, '
                'message=%s, details=%s' %
                (self.type, self.code, self.txnId, self.message, self.details))

    pass


class APIError(Exception):
    pass


class InvalidCredentialsError(APIError):
    pass


class Client(object):
    """
    The main client to be instantiated by the user.
    """
    def __init__(self, username, api_key,
                 base_url=DEFAULT_API_URL, region='us'):
        """
        @param username: Rackspace username.
        @type username: C{str}
        @param api_key: Rackspace API key.
        @type api_key: C{str}
        @param base_url: The base Cloud Registry URL.
        @type base_url: C{str}
        @param region: Rackspace region.
        @type region: C{str}
        """
        self.username = username
        self.api_key = api_key
        self.base_url = base_url
        self.region = region
        self.sessions = SessionsClient(self.base_url, self.username,
                                       self.api_key, self.region)
        self.events = EventsClient(self.base_url, self.username,
                                   self.api_key, self.region)
        self.services = ServicesClient(self.base_url, self.username,
                                       self.api_key, self.region)
        self.configuration = ConfigurationClient(self.base_url,
                                                 self.username,
                                                 self.api_key,
                                                 self.region)
        self.account = AccountClient(self.base_url, self.username,
                                     self.api_key, self.region)



# Copyright 2012 Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

from distutils.core import Command
from setuptools import setup
from subprocess import call

from utils.dist import get_packages, get_data_files

try:
    import epydoc
    has_epydoc = True
except ImportError:
    has_epydoc = False


# Commands based on Libcloud setup.py:
# https://github.com/apache/libcloud/blob/trunk/setup.py

class Pep8Command(Command):
    description = "Run pep8 script"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            import pep8
            pep8
        except ImportError:
            print ('Missing "pep8" library. You can install it using pip: '
                   'pip install pep8')
            sys.exit(1)

        cwd = os.getcwd()
        retcode = call(('pep8 %s/service_registry/' % (cwd)).split(' '))
        sys.exit(retcode)


class ApiDocsCommand(Command):
    description = "generate API documentation"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        if not has_epydoc:
            raise RuntimeError('Missing "epydoc" package!')

        os.system(
            'pydoctor'
            ' --add-package=service_registry'
            ' --project-name=service_registry'
        )


class TestCommand(Command):
    description = "Run tests"
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def _run_mock_api_server(self):
        from service_registry.test.utils import MockAPIServerRunner
        server = MockAPIServerRunner()
        server.setUp()

    def run(self):
        self._run_mock_api_server()
        cwd = os.getcwd()
        retcode = call(('python %s/service_registry/test/test_client.py' % (cwd)).split(' '))
        sys.exit(retcode)


setup(
    name='service-registry',
    version='0.1.1',
    description='Python client for Rackspace Service Registry.',
    author='Rackspace Hosting, Inc.',
    author_email='sr@rackspace.com',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Framework :: Twisted'
    ],
    cmdclass={
        'pep8': Pep8Command,
        'apidocs': ApiDocsCommand,
        'test': TestCommand
    },
    packages=get_packages('service_registry'),
    package_dir={
        'service_registry': 'service_registry',
    },
    package_data={'service_registry': get_data_files('service_registry',
                                               parent='service_registry')},
    license='Apache License (2.0)',
    url='https://github.com/racker/python-service-registry-client',
    install_requires=[
        'python-dateutil >= 2.1',
        'requests >= 0.14.1',
        'apache-libcloud >= 0.11.3'
    ]
)



#!/usr/bin/env python

# Copyright 2012 Rackspace Hosting, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import BaseHTTPServer
import os
import re

from optparse import OptionParser

mock_action = None
signature = None

HTTP_GET_PATHS = {
    '/sessions': {'fixture_path': 'sessions-get.json'},
    '/sessions/sessionId': {'fixture_path': 'sessions-sessionId-get.json'},
    '/limits': {'fixture_path': 'limits-get.json'},
    '/events': {'fixture_path': 'events-get.json'},
    '/configuration': {'fixture_path': 'configuration-get.json'},
    '/configuration/configId':
    {'fixture_path': 'configuration-configId-get.json'},
    '/services': {'fixture_path': 'services-get.json'},
    '/services/dfw1-db1':
    {'fixture_path': 'services-dfw1-db1-get.json'},
    '/services?tag=db': {'fixture_path': 'services-tag-db-get.json'},
}

HTTP_POST_PATHS = {
    '/sessions':
    {'fixture_path': 'sessions-post.json',
     'headers': {'Location': '127.0.0.1/v1.0/7777/sessions/sessionId'}},
    '/sessions/sessionId/heartbeat':
    {'fixture_path': 'sessions-sessionId-heartbeat-post.json',
     'status_code': 200},
    '/services':
    {'headers': {'Location': '127.0.0.1/v1.0/7777/services/dfw1-db1'}}
}

usage = 'usage: %prog --port=<port> --fixtures-dir=<fixtures directory>'
parser = OptionParser(usage=usage)
parser.add_option("--port", dest='port', default=8881,
                  help='Port to listen on', metavar='PORT')
parser.add_option("--fixtures-dir", dest='fixtures_dir',
                  default='fixtures/response/',
                  help='The folder in which JSON fixtures for the tests live')

(options, args) = parser.parse_args()


class Handler(BaseHTTPServer.BaseHTTPRequestHandler):

    def _read_fixture(self, path):
        fixture_path = os.path.join(options.fixtures_dir, path)
        with open(fixture_path, 'r') as f:
            fixture = f.read()

        return fixture

    def _setup_response(self, method_dict, status_code):
        split_path = re.split('(\W)', self.path)
        path = ''.join(split_path[3:])
        if path in method_dict:
            fixture_path = method_dict[path].get('fixture_path', None)
            status_code = method_dict[path].get('status_code', status_code)
            headers = method_dict[path].get('headers', None)
            body = self._read_fixture(fixture_path) if fixture_path else ''

            return self._end(status_code=status_code,
                             headers=headers,
                             body=body)

    def do_GET(self):
        return self._setup_response(HTTP_GET_PATHS, 200)

    def do_POST(self):
        return self._setup_response(HTTP_POST_PATHS, 201)

    def do_PUT(self):
        if 'sessions' in self.path:
            headers = \
                {'Location': '127.0.0.1/v1.0/7777/sessions/sessionId'}
            return self._end(status_code=204, headers=headers)
        elif 'services' in self.path:
            headers = \
                {'Location': '127.0.0.1/v1.0/7777/services/dfw1-db1'}
            return self._end(status_code=204, headers=headers)

    def do_DELETE(self):
        return self._end(status_code=204)

    def _end(self, status_code=200, headers=None, body=''):
        print 'Sending response: status_code=%s, body=%s' % (status_code, body)
        self.send_response(status_code)
        self.send_header('Content-Type', 'application/json')
        self.send_header('Content-Length', str(len(body)))
        if headers:
            for key, value in headers.iteritems():
                self.send_header(key, value)
        self.end_headers()
        self.wfile.write(body)


def main():
    server_class = BaseHTTPServer.HTTPServer
    httpd = server_class(('127.0.0.1', int(options.port)), Handler)
    print 'Mock API server listening on 127.0.0.1:%s' % (options.port)

    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        pass

    httpd.server_close()

main()
